
public final class DistanceCalculator {

	/*
	 * Distance from point (x1, y1) to point (x2, y2).
	 */
	public static double distanceFromPointToPoint(double x1, double y1, double x2, double y2) {
		return Math.sqrt(Math.pow(x1 - x2, 2) + Math.pow(y1 - y2, 2));
	}
	
	/*
	 * Distance from point (x0, y0) to line that goes through points (x1, y1) and (x2, y2).
	 */
	public static double distanceFromPointToLine(double x0, double y0, double x1, double y1, double x2, double y2) {
		if (x1 == x2) {
			return Math.abs(x0 - x1);
		}
		
		if (y1 == y2) {
			return Math.abs(y0 - y1);
		}
		
		double k = (y1 - y2) / (x1 - x2);
		
		double distance = Math.abs(k * x0 - y0 - k * x2 + y2) / Math.sqrt(k * k + 1);
		
		return distance;
	}
	
	/*
	 * Distance from point (x0, y0) to line segment between points (x1, y1) and (x2, y2).
	 */
	public static double distanceFromPointToSegment(double x0, double y0, double x1, double y1, double x2, double y2) {
		if (x1 == x2) {
			if (y0 > Math.min(y1, y2) && y0 < Math.max(y1, y2)) {
				return distanceFromPointToLine(x0, y0, x1, y1, x2, y2);
			}
			return Math.min(distanceFromPointToPoint(x0, y0, x1, y1), distanceFromPointToPoint(x0, y0, x2, y2));
		}
		
		if (y1 == y2) {
			if (x0 > Math.min(x1, x2) && x0 < Math.max(x1, x2)) {
				return distanceFromPointToLine(x0, y0, x1, y1, x2, y2);
			}
			return Math.min(distanceFromPointToPoint(x0, y0, x1, y1), distanceFromPointToPoint(x0, y0, x2, y2));
		}
		
		double k = -1 / ((y1 - y2) / (x1 - x2));
		double ym = k * x0 - k * x1 + y1;
		double yn = k * x0 - k * x2 + y2;
		
		if (y0 > Math.min(ym, yn) && y0 < Math.max(ym, yn)) {
			return distanceFromPointToLine(x0, y0, x1, y1, x2, y2);
		}
		
		return Math.min(distanceFromPointToPoint(x0, y0, x1, y1), distanceFromPointToPoint(x0, y0, x2, y2));
	}
	
	
	
}
