import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Scanner;

import javax.imageio.ImageIO;

public class RescueOperation {
	public static int inputMode;
	public static String inputFile;
	public static double HELICOPTER_HEIGHT;
	public static double TERRAIN_MAX_HEIGHT;
	public static int TERRAIN_HEIGHT = 60;
	public static int TERRAIN_WIDTH = 60;
	private int time = 0;
	private String[] directions;
	private int[] locationCounter;
	private static String outputFilename = "guards.txt";
	private static String trackFilename = "tracks.txt";
	private long startTime;
	private long endTime;
	private static String solutionFoundFile = "s.txt";

	private Terrain terrain;
	
	private ArrayList<Track> tracks;
	
	public RescueOperation() {
		this.time = 0;
		
		this.terrain = null;
		
		if (inputMode == 1) {
			this.terrain = new Terrain(this.loadRidges(new File(inputFile)));
		}
		
		if (inputMode == 2) {
			this.terrain = new Terrain(this.loadHeights(new File(inputFile)));
		}
		
		if (inputMode == 3) {
			this.terrain = new Terrain(this.getHeightsFromImage(new File(inputFile), 750, 2300));
		}
		
		this.saveTerrainToFile();

		this.loadTracks();

        this.startTimer();

		this.directions = this.terrain.getFlyingDirections(this.tracks);
        this.locationCounter = new int[this.directions.length];

		for (int i = 0; i < this.directions.length; i++) {
			if (this.directions[i] == null) {
				this.saveStringToFile(solutionFoundFile, "0");
				return;
			}
		}
		
		String output = this.getOutput();
		
		this.endTimer();
		
		System.out.println("Time used: " + this.timeUsed() + " ms.");
		
		this.saveStringToFile(outputFilename, output);
		this.saveStringToFile(solutionFoundFile, "1");
	}
	
	/* 
	 * Reads and mountain ridges from a terrain file.
	 */
	public ArrayList<Ridge> loadRidges(File terrainFile) {
		ArrayList<Ridge> ridges = new ArrayList<Ridge>();

		try {
			Scanner scanner = new Scanner(terrainFile);
			double height = Double.parseDouble(scanner.nextLine());
			double stdev = Double.parseDouble(scanner.nextLine());
			TERRAIN_MAX_HEIGHT = height;
			
			while (scanner.hasNextLine()) {
				String line = scanner.nextLine();
				String[] coordinates = line.split(" ");
				double p1x = Double.parseDouble(coordinates[0]);
				double p1y = Double.parseDouble(coordinates[1]);
				double p2x = Double.parseDouble(coordinates[2]);
				double p2y = Double.parseDouble(coordinates[3]);
				Ridge ridge = new Ridge(p1x, p1y, p2x, p2y, height, stdev);
				ridges.add(ridge);
			}
			
			scanner.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

		return ridges;
	}
	
	/* 
	 * Reads height data from a terrain file.
	 */
	public double[][] loadHeights(File heightFile) {
		double[][] heights = new double[TERRAIN_HEIGHT][TERRAIN_WIDTH];
		
		try {
			Scanner scanner = new Scanner(heightFile);
			double max = 0;
			for (int y = 0; y < heights.length; y++) {
				String[] z = scanner.nextLine().split(",");
				for (int x = 0; x < heights[0].length; x++) {
					heights[y][x] = Double.parseDouble(z[x]);
					if (heights[y][x] > max) {
						max = heights[y][x];
					}
				}
			}
			TERRAIN_MAX_HEIGHT = max;
			scanner.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return heights;
	}
	
	/* 
	 * Reads height data from a grayscale image file.
	 */
	public double[][] getHeightsFromImage(File file, double minHeight, double maxHeight) {
		double[][] heights = new double[TERRAIN_HEIGHT][TERRAIN_WIDTH];
		
		try {
			BufferedImage image = ImageIO.read(file);
			double max = 0;
			for (int y = 0; y < heights.length; y++) {
				for (int x = 0; x < heights[0].length; x++) {
					int clr = image.getRGB(x * image.getWidth() / TERRAIN_WIDTH, y * image.getHeight() / TERRAIN_HEIGHT);
					int red = (clr & 0x00ff0000) >> 16;
					int green = (clr & 0x0000ff00) >> 8;
					int blue = clr & 0x000000ff;
					
					heights[y][x] = 1.0 * (red + green + blue) / 3 / 225 * (maxHeight - minHeight) + minHeight;
					if (heights[y][x] > max) {
						max = heights[y][x];
					}
				}
			}
			TERRAIN_MAX_HEIGHT = max;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return heights;
	}
	
	public void loadTracks() {
		ArrayList<Track> tracks = new ArrayList<Track>();
		
		try {
			Scanner scanner = new Scanner(new File(trackFilename));
			String h = scanner.nextLine();
			HELICOPTER_HEIGHT = Double.parseDouble(h);
			while (scanner.hasNextLine()) {
				String line = scanner.nextLine();
				String[] lineData = line.split(" ");
				int x = Integer.parseInt(lineData[0]) - 1;
				int y = Integer.parseInt(lineData[1]) - 1;
				char d = ' ';
				if (lineData[2].equals("x+")) {
					d = 'R';
				} else if (lineData[2].equals("x-")) {
					d = 'L';
				} else if (lineData[3].equals("y+")) {
					d = 'D';
				} else if (lineData[4].equals("y-")) {
					d = 'U';
				}
				if (d == ' ') {
					System.err.println("Error in track generation.");
				}
				Track t = this.generateTrack(this.terrain.getPoint(x, y), d);
				tracks.add(t);
			}
			scanner.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		
		this.tracks = tracks;
	}
	
	public Terrain getTerrain() {
		return this.terrain;
	}
	
	public void changeHelicopterHeight(double d) {
		if (HELICOPTER_HEIGHT + d >= TERRAIN_MAX_HEIGHT) {
			HELICOPTER_HEIGHT += d;
			for (Track path : this.tracks) {
				path.getHelicopter().changeHeight(d);
			}
		}
	}
	
	public Track generateTrack(Point p, char dir) {
		ArrayList<Point> points = new ArrayList<Point>();
		Point point = p;
		while (point != null) {
			points.add(point);
			point = point.getAdjacentPoint(dir);
		}
		return new Track(points);		
	}
	
	public ArrayList<Track> getTracks() {
		return this.tracks;
	}

	public boolean endPosition() {
		for (Track t : this.tracks) {
			if (!t.getHelicopter().getPoint().equals(t.getEnd())) {
				return false;
			}
		}
		return true;
	}
	
	public boolean startPosition() {
		for (Track t : this.tracks) {
			if (!t.getHelicopter().getPoint().equals(t.getStart())) {
				return false;
			}
		}
		return true;
	}
	
	/*
	 * Next time-step.
	 * TODO: Doesn't work if the helicopter wants to move backwards.
	 */
	public void next() {
		if (this.endPosition()) {
			return;
		}
		
		int i = this.nextToMove();
		
		this.tracks.get(i).moveHelicopterForwards();
		
		int m = -1;
		int n = -1;
		
		if (i > 0) {
			m = Integer.parseInt(this.directions[i - 1].split(",")[this.locationCounter[i - 1]].substring(1));
			if (m == i) {
				this.locationCounter[i - 1]++;
			}
		}
		
		if (i < this.directions.length) {
			n = Integer.parseInt(this.directions[i].split(",")[this.locationCounter[i]].substring(1));
			if (n == i) {
				this.locationCounter[i]++;
			}
		}
		
		this.time++;
	}
	
	private int nextToMove() {
		for (int i = 0; i < this.directions.length; i++) {
			if (this.locationCounter[i] >= this.directions[i].split(",").length) {
				continue;
			}
			
			String string = this.directions[i].split(",")[this.locationCounter[i]];
			int next = Integer.parseInt(string.substring(1));
			if (i == next) {
				return i;
			}
		}
		return this.directions.length;
	}

    public static void main(String[] args) {
    
    	if (args.length > 0) {
    		inputMode = Integer.parseInt(args[0]);
    		inputFile = args[1];
    	} else {
    		inputMode = 1;
    		inputFile = "terrain.txt";
    	}
        
        new RescueOperation();
    }

    public String toString() {
    	String s = "";
    	for (int t = 0; t < this.tracks.size(); t++) {
    		Helicopter h = this.tracks.get(t).getHelicopter();
    		Point p = h.getPoint();
    		s += (this.time + 1) + "," + (t + 1) + "," + (p.getX() + 1) + "," + (p.getY() + 1) + "," + h.getHeight() + "\n";
    	}
    	return s;
    }
    
    public String getOutput() {
    	String output = "";
    	while (!this.endPosition()) {
    		output += this;
    		this.next();	
    	}
    	output += this;
    	return output;
    }

    public void saveStringToFile(String filename, String string) {
    	try {
    		FileWriter fstream = new FileWriter(filename);
    		BufferedWriter out = new BufferedWriter(fstream);
    		out.write(string);
    		out.close();
    	} catch (Exception e) {
    		System.err.println("Error: " + e.getMessage());
    	}
    }
    
	
	public void saveTerrainToFile() {
    	try {
    		FileWriter fstream = new FileWriter("terrainZ.txt");
    		BufferedWriter out = new BufferedWriter(fstream);
    		out.write(this.terrain.toString());
    		out.close();
    	} catch (Exception e) {
    		System.err.println("Error: " + e.getMessage());
    	}
	}
	
	public void startTimer() {
		this.startTime = System.currentTimeMillis();
	}
	
	public void endTimer() {
		this.endTime = System.currentTimeMillis();
	}
	
	public long timeUsed() {
		return this.endTime - this.startTime;
	}
}
