clear; close all;
if exist('guards.txt', 'file')==2
  delete('guards.txt');
end
if exist('s.txt', 'file')==2
  delete('s.txt');
end

% Change the inputMode variable to indicate what kind of data file you
% would like to use for terrain generation:
% - If you have the terrain data as a text file containing normally distributed
%       mountain ridges and coordinates of their end points, set inputMode = 1.
% - If you have the terrain data as a 60x60 file of heights, set inputMode = 2. 
% - If you have the terrain data as a grayscale image, set inputMode = 3.
%       Using this will divide the terrain into 60x60 grid,
%       setting the grid square's elevation equal to the rgb value of the 
%       shade of gray of the corresponding pixel.
% Change the inputFile to the filename of the terrain data file.
inputMode = 1;
inputFile = 'terrain.txt';

%system('javac RescueOperation.java'); % compiling the Java code

disp('Generating terrain and computing trajectories...')
system(['java RescueOperation ' num2str(inputMode) ' ' inputFile]);

solutionFound = load('s.txt');

if solutionFound
    disp('Success (trajectories found)!')
    disp('Visualization in progress...')
else
    disp('Sorry, cannot sweep the terrain.')
    disp('Increase elevation (first line of tracks.txt)')
    disp('and/or reposition tracks and/or add more tracks.')
end

%%%%%%%%%%% TERRAIN %%%%%%%%%%
Z = loadTerrain(inputFile);
%Z = load('terrainZ.txt'); % terrain that has been generated by Java

h = max(max(Z));
%%%%%%%%%% /TERRAIN %%%%%%%%%%


%%%%%%%%%% HELICOPTERS %%%%%%%%%%
if solutionFound
    H = load('guards.txt');

    helicopterheight = H(1, 5);

    nHelicopters = max(H(:, 2));
end
%%%%%%%%%% /HELICOPTERS %%%%%%%%%%


%%%%%%%%%% TERRAIN VISUALIZATION %%%%%%%%%%
surf(Z')
hold on
if solutionFound
    zlim([0 1.5*helicopterheight])
else
    zlim([0 1.5*h])
end

c = 0:0.01:1;
c = c .* h;
r1 = 13;
g1 = 115;
b1 = 0;
r2 = 69;
g2 = 40;
b2 = 0;

C = [r1 + (r2 - r1) / h * c; ...
     g1 + (g2 - g1) / h * c; ...
     b1 + (b2 - b1) / h * c]' / 225;
colormap(C);
%%%%%%%%%% /TERRAIN VISUALIZATION %%%%%%%%%%

view(-60, 54);

if solutionFound
    %
else
    return;
end

%%%%%%%%%% VIDEO %%%%%%%%%%
nFrames = size(H, 1) / nHelicopters;
mov = moviein(nFrames);
for t = 1:nFrames
    hels = H(H(:,1)==t,:);
    ss = scatter3(hels(:, 3), hels(:, 4), hels(:, 5), 'b', 'fill');
    ll = [];
    for h = 1:size(hels, 1) - 1
        x1 = hels(h, 3);
        y1 = hels(h, 4);
        z1 = 0;
        x2 = hels(h + 1, 3);
        y2 = hels(h + 1, 4);
        z2 = helicopterheight;
        
        p1 = [x1 y1 z2];
        p2 = [x1 y1 z1];
        p3 = [x2 y2 z1];
        p4 = [x2 y2 z2];
        
        ll(h) = fill3([p1(1) p2(1) p3(1) p4(1)], ...
                      [p1(2) p2(2) p3(2) p4(2)], ...
                      [p1(3) p2(3) p3(3) p4(3)], ...
                      [1 1 0], 'FaceAlpha', 0.5);
    end
    
    mov(:, t) = getframe(gcf);
    
    %t
    %pause;
    
    delete(ss);
    delete(ll);
end
disp('Saving video to m.avi...')
close(gcf)
movie2avi(mov,'m.avi','quality',100,'compression','None');
%%%%%%%%%% /VIDEO %%%%%%%%%%
